package cn.com.poc.expose.aggregate.impl;

import cn.com.poc.common.utils.Assert;
import cn.com.poc.common.utils.DateUtils;
import cn.com.poc.data_analyze.constants.DataAnalyzeTimeRangeEnum;
import cn.com.poc.data_analyze.domain.DataAnalyzeTimeRange;
import cn.com.poc.data_analyze.query.MemberPointUsageOverallQueryCondition;
import cn.com.poc.data_analyze.query.MemberPointUsageOverallQueryItem;
import cn.com.poc.data_analyze.query.MemberPointUsageTrendQueryCondition;
import cn.com.poc.data_analyze.query.MemberPointUsageTrendQueryItem;
import cn.com.poc.data_analyze.service.BizDataAnalyzePointRecordService;
import cn.com.poc.expose.aggregate.DataStatisticsService;
import cn.com.poc.expose.dto.data_statistics.PlatformPointUsageDto;
import cn.com.poc.expose.entity.PlatformPointUsageEntity;
import cn.com.yict.framemax.core.i18n.I18nMessageException;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author alex.yao
 * @date 2024/12/23
 */
@Service
public class DataStatisticsServiceImpl implements DataStatisticsService {

    @Resource
    private BizDataAnalyzePointRecordService bizDataAnalyzePointRecordService;

    @Override
    public PlatformPointUsageEntity platformPointUsage(Long memberId, List<String> channels) {
        MemberPointUsageOverallQueryCondition memberPointUsageOverallQueryCondition = new MemberPointUsageOverallQueryCondition();
        memberPointUsageOverallQueryCondition.setMemberId(memberId);
        memberPointUsageOverallQueryCondition.setChannels(channels);
        List<MemberPointUsageOverallQueryItem> items = bizDataAnalyzePointRecordService.memberPointUsageOverallQuery(memberPointUsageOverallQueryCondition, null);
        PlatformPointUsageEntity platformPointUsageEntity = new PlatformPointUsageEntity();
        if (CollectionUtils.isNotEmpty(items)) {
            MemberPointUsageOverallQueryItem memberPointUsageOverallQueryItem = items.get(0);
            platformPointUsageEntity.setToday(memberPointUsageOverallQueryItem.getTodayTotalPoints());
            platformPointUsageEntity.setWeek(memberPointUsageOverallQueryItem.getWeekTotalPoints());
            platformPointUsageEntity.setMonth(memberPointUsageOverallQueryItem.getMonthTotalPoints());
            platformPointUsageEntity.setYear(memberPointUsageOverallQueryItem.getYearTotalPoints());
        } else {
            platformPointUsageEntity.setToday(0D);
            platformPointUsageEntity.setWeek(0D);
            platformPointUsageEntity.setMonth(0D);
            platformPointUsageEntity.setYear(0D);
        }
        return platformPointUsageEntity;
    }

    @Override
    public Map<String, List<MemberPointUsageTrendQueryItem>> platformPointTrend(Long memberId, List<String> channels, DataAnalyzeTimeRange timeRange) {
        Map<String, List<MemberPointUsageTrendQueryItem>> result = new TreeMap<>();
        String startTime;
        String endTime;
        Date startDate;
        Date endDate;
        switch (DataAnalyzeTimeRangeEnum.getByType(timeRange.getRangType())) {
            case week:
                endDate = DateUtils.getTodayEnd();
                startDate = DateUtils.getDayStartTime(DateUtils.addDays(endDate, -6));
                startTime = DateUtils.formatDate(startDate, DateUtils.yyyy_MM_dd);
                endTime = DateUtils.formatDate(endDate, DateUtils.yyyy_MM_dd_HH_mm_ss);
                break;
            case month:
                endDate = DateUtils.getTodayEnd();
                startDate = DateUtils.getDayStartTime(DateUtils.addMonth(endDate, -1));
                startTime = DateUtils.formatDate(startDate, DateUtils.yyyy_MM_dd);
                endTime = DateUtils.formatDate(endDate, DateUtils.yyyy_MM_dd_HH_mm_ss);
                break;
            case customize:
                Assert.notBlank(timeRange.getStartTime());
                Assert.notBlank(timeRange.getEndTime());
                startTime = timeRange.getStartTime();
                endDate = DateUtils.getDayEnd(DateUtils.stringToDateShort(timeRange.getEndTime()));
                endTime = DateUtils.formatDate(endDate, DateUtils.yyyy_MM_dd_HH_mm_ss);
                break;
            default:
                throw new I18nMessageException("data-analyze/not.support.rang.type");
        }
        for (String channel : channels) {
            MemberPointUsageTrendQueryCondition memberPointUsageTrendQueryCondition = new MemberPointUsageTrendQueryCondition();
            memberPointUsageTrendQueryCondition.setStartDate(startTime);
            memberPointUsageTrendQueryCondition.setEndDate(endTime);
            memberPointUsageTrendQueryCondition.setMemberId(memberId);
            memberPointUsageTrendQueryCondition.setChannel(channel);
            List<MemberPointUsageTrendQueryItem> items = bizDataAnalyzePointRecordService.memberPointUsageTrendQuery(memberPointUsageTrendQueryCondition, null);
            result.put(channel, items);
        }
        return result;
    }
}
